@@ -4,7 +4,7 @@ from ihm import AsymUnitRange, _remove_identical  # noqa: F401
 import modelcif.data
 import sys
 
-__version__ = '0.5'
+__version__ = '0.7'
 
 
 class System(object):
@@ -175,8 +175,22 @@ class System(object):
                 else:
                     for s in sg:
                         yield s
+
+        def _all_entities():
+            return itertools.chain(
+                self.entities, (t.entity for t in self.templates))
+
+        def _all_descriptor_software():
+            comps = frozenset(comp for e in _all_entities()
+                              for comp in e.sequence)
+            for comp in comps:
+                if hasattr(comp, 'descriptors') and comp.descriptors:
+                    for desc in comp.descriptors:
+                        if desc.software:
+                            yield desc.software
         return (itertools.chain(
-            self.software, _all_software_in_groups()))
+            self.software, _all_software_in_groups(),
+            _all_descriptor_software()))
 
     def _all_assemblies(self):
         """Iterate over all Assemblies in the system.
@@ -222,8 +236,10 @@ class System(object):
         """Return all DataGroup (or singleton Data) objects"""
         return itertools.chain(
             self.data_groups,
-            (step.input_data for p in self.protocols for step in p.steps),
-            (step.output_data for p in self.protocols for step in p.steps))
+            (step.input_data for p in self.protocols for step in p.steps
+             if step.input_data),
+            (step.output_data for p in self.protocols for step in p.steps
+             if step.output_data))
 
     def _all_software_groups(self):
         """Return all SoftwareGroup (or singleton Software) objects"""
@@ -304,6 +320,11 @@ This is implemented as a simple list of asymmetric units (or parts of
 them), i.e. a list of :class:`AsymUnit` and/or :class:`AsymUnitRange`
 objects. An Assembly is typically passed to the :class:`modelcif.model.Model`
 constructor.
+
+Note that the ModelCIF dictionary has deprecated the corresponding
+``ma_struct_assembly`` category, so any name or description of the assembly
+will not be written to the mmCIF file. The ModelCIF dictionary requires that
+all models have the same composition.
 """
 
 
@@ -357,8 +378,8 @@ class SoftwareParameter(object):
         self.description = description
 
     def __repr__(self):
-        return("<SoftwareParameter(name=%r, value=%r)>"
-               % (self.name, self.value))
+        return ("<SoftwareParameter(name=%r, value=%r)>"
+                % (self.name, self.value))
 
 
 class Transformation(object):
