@@ -20,8 +20,8 @@ class _AuditConformDumper(Dumper):
         with writer.category("_audit_conform") as lp:
             # Update to match the version of the ModelCIF dictionary
             # we support:
-            lp.write(dict_name="mmcif_ma.dic", dict_version="1.4.0",
-                     dict_location=self.URL % "a24fcfa")
+            lp.write(dict_name="mmcif_ma.dic", dict_version="1.4.4",
+                     dict_location=self.URL % "330c8af")
 
 
 class _EntryLinkDumper(Dumper):
@@ -30,12 +30,6 @@ class _EntryLinkDumper(Dumper):
             lp.write(id=1, entry_id=system.id, details=system.entry_details)
 
 
-class _ExptlDumper(Dumper):
-    def dump(self, system, writer):
-        with writer.category("_exptl") as lp:
-            lp.write(entry_id=system.id, method='THEORETICAL MODEL')
-
-
 class _DatabaseDumper(Dumper):
     def dump(self, system, writer):
         if system.database:
@@ -49,20 +43,62 @@ class _ChemCompDumper(Dumper):
     # components referenced only by Templates, as their Entities are not
     # included in system.entities by default
 
+    _prov_map = {'core': 'CCD Core', 'ma': 'CCD MA', 'local': 'CCD local'}
+
     def _get_entities(self, system):
         return itertools.chain(
             system.entities, (t.entity for t in system.templates))
 
+    def _get_provenance(self, comp):
+        # Older python-ihm does not provide ChemComp.ccd
+        ccd = comp.ccd if hasattr(comp, 'ccd') else None
+        if ccd is None:
+            ccd = 'core'
+            if hasattr(comp, 'descriptors') and comp.descriptors:
+                ccd = 'local'
+        val = self._prov_map.get(ccd)
+        if not val:
+            raise KeyError("Invalid ccd value %s for %s; can be %s, or None"
+                           % (repr(comp.ccd), comp,
+                              ", ".join(sorted(self._prov_map.keys()))))
+        return val
+
     def dump(self, system, writer):
         comps = frozenset(
             comp for e in self._get_entities(system) for comp in e.sequence)
 
         with writer.loop("_chem_comp", ["id", "type", "name",
-                                        "formula", "formula_weight"]) as lp:
+                                        "formula", "formula_weight",
+                                        "ma_provenance"]) as lp:
             for comp in sorted(comps, key=operator.attrgetter('id')):
                 lp.write(id=comp.id, type=comp.type, name=comp.name,
                          formula=comp.formula,
-                         formula_weight=comp.formula_weight)
+                         formula_weight=comp.formula_weight,
+                         ma_provenance=self._get_provenance(comp))
+
+
+class _ChemCompDescriptorDumper(Dumper):
+    def _get_entities(self, system):
+        return itertools.chain(
+            system.entities, (t.entity for t in system.templates))
+
+    def dump(self, system, writer):
+        ordinal = itertools.count(1)
+        comps = frozenset(
+            comp for e in self._get_entities(system) for comp in e.sequence)
+
+        with writer.loop("_ma_chem_comp_descriptor",
+                         ["ordinal_id", "chem_comp_id", "chem_comp_name",
+                          "type", "value", "details", "software_id"]) as lp:
+            for comp in sorted(comps, key=operator.attrgetter('id')):
+                if not hasattr(comp, 'descriptors') or not comp.descriptors:
+                    continue
+                for desc in comp.descriptors:
+                    lp.write(ordinal_id=next(ordinal), chem_comp_id=comp.id,
+                             chem_comp_name=comp.name, type=desc.type,
+                             value=desc.value, details=desc.details,
+                             software_id=desc.software._id
+                             if desc.software else None)
 
 
 class _TargetRefDBDumper(Dumper):
@@ -276,40 +312,6 @@ class _DataRefDBDumper(Dumper):
                          if d.release_date else None)
 
 
-class _AssemblyDumper(Dumper):
-    def finalize(self, system):
-        for n, asmb in enumerate(system.assemblies):
-            asmb._id = n + 1
-
-    def dump(self, system, writer):
-        self.dump_summary(system, writer)
-        self.dump_details(system, writer)
-
-    def dump_summary(self, system, writer):
-        ordinal = itertools.count(1)
-        with writer.loop("_ma_struct_assembly",
-                         ["ordinal_id", "assembly_id", "entity_id", "asym_id",
-                          "seq_id_begin", "seq_id_end"]) as lp:
-            for a in system.assemblies:
-                for comp in a:
-                    entity = comp.entity if hasattr(comp, 'entity') else comp
-                    lp.write(
-                        ordinal_id=next(ordinal), assembly_id=a._id,
-                        entity_id=entity._id,
-                        asym_id=comp._id if hasattr(comp, 'entity') else None,
-                        seq_id_begin=comp.seq_id_range[0],
-                        seq_id_end=comp.seq_id_range[1])
-
-    def dump_details(self, system, writer):
-        with writer.loop("_ma_struct_assembly_details",
-                         ["assembly_id", "assembly_name",
-                          "assembly_description"]) as lp:
-            for a in system.assemblies:
-                lp.write(assembly_id=a._id,
-                         assembly_name=a.name,
-                         assembly_description=a.description)
-
-
 class _TemplateTransformDumper(Dumper):
     def finalize(self, system):
         for n, trans in enumerate(system.template_transformations):
@@ -446,12 +448,15 @@ class _AlignmentDumper(Dumper):
         with writer.loop(
                 "_ma_template_ref_db_details",
                 ["template_id", "db_name", "db_name_other_details",
-                 "db_accession_code"]) as lp:
+                 "db_accession_code", "db_version_date"]) as lp:
             for tmpl in system.templates:
                 for ref in tmpl.references:
                     lp.write(template_id=tmpl._id, db_name=ref.name,
                              db_name_other_details=ref.other_details,
-                             db_accession_code=ref.accession)
+                             db_accession_code=ref.accession,
+                             db_version_date=date.isoformat(
+                                 ref.db_version_date)
+                             if ref.db_version_date else None)
 
     def dump_target_template_poly_mapping(self, system, writer):
         ordinal = itertools.count(1)
@@ -550,8 +555,10 @@ class _ProtocolDumper(Dumper):
                              step_name=s.name, details=s.details,
                              software_group_id=s.software._group_id
                              if s.software else None,
-                             input_data_group_id=s.input_data._data_group_id,
-                             output_data_group_id=s.output_data._data_group_id)
+                             input_data_group_id=s.input_data._data_group_id
+                             if s.input_data else None,
+                             output_data_group_id=s.output_data._data_group_id
+                             if s.output_data else None)
 
 
 class _ModelDumper(ihm.dumper._ModelDumperBase):
@@ -564,7 +571,7 @@ class _ModelDumper(ihm.dumper._ModelDumperBase):
         ordinal = itertools.count(1)
         with writer.loop("_ma_model_list",
                          ["ordinal_id", "model_id", "model_group_id",
-                          "model_name", "model_group_name", "assembly_id",
+                          "model_name", "model_group_name",
                           "data_id", "model_type",
                           "model_type_other_details"]) as lp:
             for group in system.model_groups:
@@ -572,7 +579,6 @@ class _ModelDumper(ihm.dumper._ModelDumperBase):
                     lp.write(ordinal_id=next(ordinal), model_id=model._id,
                              model_group_id=group._id, model_name=model.name,
                              model_group_name=group.name,
-                             assembly_id=model.assembly._id,
                              data_id=model._data_id,
                              model_type=model.model_type,
                              model_type_other_details=model.other_details)
@@ -780,11 +786,11 @@ class ModelCIFVariant(Variant):
        See :func:`write` and :class:`ihm.dumper.Variant`."""
     _dumpers = [
         ihm.dumper._EntryDumper,  # must be first
-        ihm.dumper._StructDumper, _ExptlDumper, ihm.dumper._CommentDumper,
+        ihm.dumper._StructDumper, ihm.dumper._CommentDumper,
         _AuditConformDumper, _DatabaseDumper, ihm.dumper._CitationDumper,
         ihm.dumper._SoftwareDumper, _SoftwareGroupDumper,
         ihm.dumper._AuditAuthorDumper,
-        ihm.dumper._GrantDumper, _ChemCompDumper,
+        ihm.dumper._GrantDumper, _ChemCompDumper, _ChemCompDescriptorDumper,
         ihm.dumper._EntityDumper,
         ihm.dumper._EntitySrcGenDumper, ihm.dumper._EntitySrcNatDumper,
         ihm.dumper._EntitySrcSynDumper, _TargetRefDBDumper,
@@ -792,8 +798,7 @@ class ModelCIFVariant(Variant):
         ihm.dumper._EntityPolySeqDumper, ihm.dumper._StructAsymDumper,
         ihm.dumper._PolySeqSchemeDumper, ihm.dumper._NonPolySchemeDumper,
         _DataDumper, _DataGroupDumper, _DataRefDBDumper,
-        _TargetEntityDumper, _AssemblyDumper,
-        _TemplateTransformDumper, _AlignmentDumper,
+        _TargetEntityDumper, _TemplateTransformDumper, _AlignmentDumper,
         _ProtocolDumper, _ModelDumper, _AssociatedDumper, _QAMetricDumper]
 
     def get_dumpers(self):
@@ -804,8 +809,15 @@ class ModelCIFVariant(Variant):
         # associated files (the rest use the default writer)
         category_map = {}
         copy_category_map = {}
-        for r in system.repositories:
+
+        def _all_repo_files(r):
             for f in r.files:
+                yield f
+                if hasattr(f, 'files'):
+                    for subf in f.files:
+                        yield subf
+        for r in system.repositories:
+            for f in _all_repo_files(r):
                 if (not hasattr(f, 'categories')
                         or (not f.categories and not f.copy_categories)):
                     continue
