@@ -7,6 +7,7 @@ import modelcif.qa_metric
 import modelcif.alignment
 import modelcif.reference
 import modelcif.associated
+import modelcif.descriptor
 import ihm
 import ihm.source
 import ihm.reader
@@ -150,6 +151,53 @@ class _DatabaseHandler(Handler):
             id=database_id, code=database_code)
 
 
+class _ChemCompHandler(Handler):
+    """Similar to ihm.reader._ChemCompHandler but also handles
+       the ma_provenance data item"""
+    category = '_chem_comp'
+
+    _prov_map = {'ccd core': 'core', 'ccd ma': 'ma', 'ccd local': 'local'}
+
+    def __init__(self, *args):
+        super(_ChemCompHandler, self).__init__(*args)
+        # Map _chem_comp.type to corresponding subclass of ihm.ChemComp
+        self.type_map = dict((x[1].type.lower(), x[1])
+                             for x in inspect.getmembers(ihm, inspect.isclass)
+                             if issubclass(x[1], ihm.ChemComp))
+
+    def __call__(self, type, id, name, formula, ma_provenance):
+        typ = 'other' if type is None else type.lower()
+        s = self.sysr.chem_comps.get_by_id(
+            id, self.type_map.get(typ, ihm.ChemComp))
+        self.copy_if_present(s, locals(), keys=('name', 'formula'))
+        if ma_provenance:
+            s.ccd = self._prov_map.get(ma_provenance.lower())
+
+
+class _ChemCompDescriptorHandler(Handler):
+    category = '_ma_chem_comp_descriptor'
+
+    def __init__(self, *args):
+        super(_ChemCompDescriptorHandler, self).__init__(*args)
+        # Map _chem_comp_descriptor.type to corresponding subclass of
+        # modelcif.descriptor.Descriptor
+        self._type_map = dict(
+            (x[1].type.lower(), x[1])
+            for x in inspect.getmembers(modelcif.descriptor, inspect.isclass)
+            if issubclass(x[1], modelcif.descriptor.Descriptor)
+            and x[1] is not modelcif.descriptor.Descriptor)
+
+    def __call__(self, chem_comp_id, type, value, details, software_id):
+        s = self.sysr.chem_comps.get_by_id(chem_comp_id)
+        type_class = self._type_map.get(
+            type.lower(), modelcif.descriptor.Descriptor)
+        software = self.sysr.software.get_by_id_or_none(software_id)
+        desc = type_class(value=value, details=details, software=software)
+        if not hasattr(s, 'descriptors') or not s.descriptors:
+            s.descriptors = []
+        s.descriptors.append(desc)
+
+
 class _EntityNonPolyHandler(Handler):
     category = '_pdbx_entity_nonpoly'
     _mmmap = {'explicit': True, 'implicit': False}
@@ -157,7 +205,11 @@ class _EntityNonPolyHandler(Handler):
     def __call__(self, entity_id, comp_id, ma_model_mode):
         s = self.sysr.entities.get_by_id(entity_id)
         s.sequence = (self.sysr.chem_comps.get_by_id(comp_id),)
-        self.sysr.ma_model_mode_map[s] = self._mmmap.get(ma_model_mode.lower())
+        if ma_model_mode in (None, ihm.unknown):
+            self.sysr.ma_model_mode_map[s] = ma_model_mode
+        else:
+            self.sysr.ma_model_mode_map[s] = self._mmmap.get(
+                ma_model_mode.lower())
 
 
 class _TemplatePolyHandler(Handler):
@@ -313,7 +365,7 @@ class _EnumerationMapper(object):
             self._map[name] = ExtraType
             return ExtraType
         # If name is "Other" then treat other_details as the key
-        other_det_up = other_det.upper()
+        other_det_up = other_det if other_det is None else other_det.upper()
         if other_det_up not in self._other_map:
             class CustomType(self._base_class):
                 other_details = other_det
@@ -400,10 +452,11 @@ class _TemplateRefDBHandler(Handler):
             modelcif.reference, modelcif.reference.TemplateReference)
 
     def __call__(self, template_id, db_name, db_name_other_details,
-                 db_accession_code):
+                 db_accession_code, db_version_date):
         t = self.sysr.templates.get_by_id(template_id)
         typ = self.type_map.get(db_name, db_name_other_details)
-        ref = typ(accession=db_accession_code)
+        ref = typ(accession=db_accession_code,
+                  db_version_date=_get_date(db_version_date))
         t.references.append(ref)
 
 
@@ -814,7 +867,8 @@ class ModelCIFVariant(Variant):
         ihm.reader._StructHandler, ihm.reader._SoftwareHandler,
         ihm.reader._CitationHandler, ihm.reader._AuditAuthorHandler,
         ihm.reader._GrantHandler, ihm.reader._CitationAuthorHandler,
-        ihm.reader._ChemCompHandler, ihm.reader._EntityHandler,
+        _ChemCompHandler, _ChemCompDescriptorHandler,
+        ihm.reader._EntityHandler,
         ihm.reader._EntitySrcNatHandler, ihm.reader._EntitySrcGenHandler,
         ihm.reader._EntitySrcSynHandler, ihm.reader._EntityPolyHandler,
         ihm.reader._EntityPolySeqHandler, _EntityNonPolyHandler,
