@@ -22,6 +22,7 @@ import modelcif.model
 import modelcif.reference
 import modelcif.alignment
 import modelcif.associated
+import modelcif.descriptor
 import ihm.format
 import ihm.dumper
 
@@ -74,14 +75,6 @@ class Tests(unittest.TestCase):
         self.assertEqual(out, "_database_2.database_code bar\n"
                               "_database_2.database_id foo\n")
 
-    def test_exptl_dumper(self):
-        """Test ExptlDumper"""
-        system = modelcif.System(id='foo')
-        dumper = modelcif.dumper._ExptlDumper()
-        out = _get_dumper_output(dumper, system)
-        self.assertEqual(out, "_exptl.entry_id foo\n"
-                              "_exptl.method 'THEORETICAL MODEL'\n")
-
     def test_software_group_dumper(self):
         """Test SoftwareGroupDumper"""
         class MockObject(object):
@@ -377,6 +370,8 @@ _ma_qa_metric_local_pairwise.metric_value
             input_data=indat, output_data=outdat))
         p.steps.append(modelcif.protocol.ModelingStep(
             name='modstep', input_data=indat, output_data=outdat))
+        p.steps.append(modelcif.protocol.ModelingStep(
+            name='nullstep', input_data=None, output_data=None))
         system.protocols.append(p)
         dumper = modelcif.dumper._ProtocolDumper()
         dumper.finalize(system)
@@ -394,6 +389,7 @@ _ma_protocol_step.input_data_group_id
 _ma_protocol_step.output_data_group_id
 1 1 1 'template search' tsstep 'some details' 42 1 2
 2 1 2 modeling modstep . . 1 2
+3 1 3 modeling nullstep . . . .
 #
 """)
 
@@ -438,14 +434,13 @@ _ma_model_list.model_id
 _ma_model_list.model_group_id
 _ma_model_list.model_name
 _ma_model_list.model_group_name
-_ma_model_list.assembly_id
 _ma_model_list.data_id
 _ma_model_list.model_type
 _ma_model_list.model_type_other_details
-1 1 1 'test model' 'test group' 2 42 'Homology model' .
-2 2 1 model2 'test group' 2 43 'Ab initio model' .
-3 3 1 model3 'test group' 2 44 Other 'custom model'
-4 1 2 'test model' 'second group' 2 42 'Homology model' .
+1 1 1 'test model' 'test group' 42 'Homology model' .
+2 2 1 model2 'test group' 43 'Ab initio model' .
+3 3 1 model3 'test group' 44 Other 'custom model'
+4 1 2 'test model' 'second group' 42 'Homology model' .
 #
 #
 loop_
@@ -537,12 +532,17 @@ _ma_target_ref_db_details.seq_db_sequence_checksum
         asym = modelcif.AsymUnit(tgt_e, id='A')
         asym._id = 'A'
         system.asym_units.append(asym)
-        ref1 = modelcif.reference.PDB('1abc')
+        ref1 = modelcif.reference.PDB('1abc',
+                                      db_version_date=date(1979, 11, 22))
         ref2 = CustomRef('2xyz')
+        ref3 = modelcif.reference.PubChem("1234")
+        ref4 = modelcif.reference.AlphaFoldDB("P12345",
+                                              db_version_date=date(2022, 6, 1))
         tr = modelcif.Transformation.identity()
         tr._id = 42
         t = modelcif.Template(tmp_e, asym_id='H', model_num=1, name='testtmp',
-                              transformation=tr, references=[ref1, ref2],
+                              transformation=tr,
+                              references=[ref1, ref2, ref3, ref4],
                               strand_id='Z')
         t._data_id = 99
         p = modelcif.alignment.Pair(
@@ -553,9 +553,18 @@ _ma_target_ref_db_details.seq_db_sequence_checksum
         aln = Alignment(name='testaln', pairs=[p])
         aln._data_id = 100
         system.alignments.append(aln)
+        # The same alignment using HHblits e-value
+        p = modelcif.alignment.Pair(
+            template=p.template,
+            target=p.target,
+            score=modelcif.alignment.HHblitsEValue("1e-14"),
+            identity=p.identity)
+        aln = Alignment(name='testaln', pairs=[p])
+        aln._data_id = 101
+        system.alignments.append(aln)
         # Alignment with no pairs
         aln2 = Alignment(name='testaln2', pairs=[])
-        aln2._data_id = 101
+        aln2._data_id = 102
         system.alignments.append(aln2)
         system._before_write()  # populate system.templates
 
@@ -576,6 +585,7 @@ _ma_template_details.template_label_entity_id
 _ma_template_details.template_model_num
 _ma_template_details.template_auth_asym_id
 1 1 'reference database' polymer 42 99 A H . 1 Z
+2 1 'reference database' polymer 42 99 A H . 1 Z
 #
 #
 loop_
@@ -598,8 +608,11 @@ _ma_template_ref_db_details.template_id
 _ma_template_ref_db_details.db_name
 _ma_template_ref_db_details.db_name_other_details
 _ma_template_ref_db_details.db_accession_code
-1 PDB . 1abc
-1 Other 'my custom ref' 2xyz
+_ma_template_ref_db_details.db_version_date
+1 PDB . 1abc 1979-11-22
+1 Other 'my custom ref' 2xyz .
+1 PubChem . 1234 .
+1 AlphaFoldDB . P12345 2022-06-01
 #
 #
 loop_
@@ -609,6 +622,7 @@ _ma_target_template_poly_mapping.target_asym_id
 _ma_target_template_poly_mapping.target_seq_id_begin
 _ma_target_template_poly_mapping.target_seq_id_end
 1 1 A 1 3
+2 1 A 1 3
 #
 #
 loop_
@@ -619,7 +633,8 @@ _ma_alignment_info.alignment_length
 _ma_alignment_info.alignment_type
 _ma_alignment_info.alignment_mode
 1 100 . 4 'target-template pairwise alignment' global
-2 101 . . 'target-template pairwise alignment' global
+2 101 . 4 'target-template pairwise alignment' global
+3 102 . . 'target-template pairwise alignment' global
 #
 #
 loop_
@@ -634,6 +649,7 @@ _ma_alignment_details.percent_sequence_identity
 _ma_alignment_details.sequence_identity_denominator
 _ma_alignment_details.sequence_identity_denominator_other_details
 1 1 1 A 'BLAST e-value' . 1e-15 42.000 'Length of the shorter sequence' .
+2 2 1 A 'HHblits e-value' . 1e-14 42.000 'Length of the shorter sequence' .
 #
 #
 loop_
@@ -643,6 +659,8 @@ _ma_alignment.target_template_flag
 _ma_alignment.sequence
 1 1 1 ACE-
 2 1 2 AC-G
+3 2 1 ACE-
+4 2 2 AC-G
 #
 """)
 
@@ -820,42 +838,6 @@ _ma_target_entity_instance.entity_id
 _ma_target_entity_instance.details
 X 42 foo
 #
-""")
-
-    def test_assembly_dumper(self):
-        """Test AssemblyDumper"""
-        system = modelcif.System()
-        e1 = modelcif.Entity('ACGT')
-        e1._id = 42
-        system.entities.append(e1)
-        asym = modelcif.AsymUnit(e1, 'foo')
-        system.asym_units.append(asym)
-        asmb = modelcif.Assembly((asym,), name='foo', description='bar')
-        system.assemblies.append(asmb)
-
-        dumper = ihm.dumper._StructAsymDumper()  # Assign _ordinal_id
-        dumper.finalize(system)
-
-        dumper = modelcif.dumper._AssemblyDumper()
-        dumper.finalize(system)
-        out = _get_dumper_output(dumper, system)
-        self.assertEqual(out, """#
-loop_
-_ma_struct_assembly.ordinal_id
-_ma_struct_assembly.assembly_id
-_ma_struct_assembly.entity_id
-_ma_struct_assembly.asym_id
-_ma_struct_assembly.seq_id_begin
-_ma_struct_assembly.seq_id_end
-1 1 42 A 1 4
-#
-#
-loop_
-_ma_struct_assembly_details.assembly_id
-_ma_struct_assembly_details.assembly_name
-_ma_struct_assembly_details.assembly_description
-1 foo bar
-#
 """)
 
     def test_associated_dumper(self):
@@ -912,7 +894,7 @@ _ma_associated_archive_file_details.description
 
         f = modelcif.associated.CIFFile(
             path='test_write_associated.cif',
-            categories=['exptl', '_AUDIT_CONFORM'],
+            categories=['struct', '_AUDIT_CONFORM'],
             entry_details='test details', entry_id='testcif')
         f2 = modelcif.associated.File(path='foo.txt', details='test file')
         r = modelcif.associated.Repository(url_root='https://example.com',
@@ -925,10 +907,36 @@ _ma_associated_archive_file_details.description
         with open('test_write_associated.cif') as fh:
             assoc_file = fh.read()
         os.unlink('test_write_associated.cif')
-        # exptl and audit_conform categories should be in associated file,
+        # struct and audit_conform categories should be in associated file,
         # not the main file
-        self.assertIn('_exptl.entry_id', assoc_file)
-        self.assertNotIn('_exptl.entry_id', main_file)
+        self.assertIn('_struct.title', assoc_file)
+        self.assertNotIn('_struct.title', main_file)
+        self.assertIn('_audit_conform.dict_name', assoc_file)
+        self.assertNotIn('_audit_conform.dict_name', main_file)
+
+    def test_write_associated_in_zip(self):
+        """Test write() function with associated files in a ZipFile"""
+        s = modelcif.System(id='system1')
+
+        f = modelcif.associated.CIFFile(
+            path='test_write_associated_in_zip.cif',
+            categories=['struct', '_AUDIT_CONFORM'],
+            entry_details='test details', entry_id='testcif')
+        zf = modelcif.associated.ZipFile(path='t.zip', files=[f])
+        r = modelcif.associated.Repository(url_root='https://example.com',
+                                           files=[zf])
+        s.repositories.append(r)
+
+        fh = StringIO()
+        modelcif.dumper.write(fh, [s])
+        main_file = fh.getvalue()
+        with open('test_write_associated_in_zip.cif') as fh:
+            assoc_file = fh.read()
+        os.unlink('test_write_associated_in_zip.cif')
+        # struct and audit_conform categories should be in associated file,
+        # not the main file
+        self.assertIn('_struct.title', assoc_file)
+        self.assertNotIn('_struct.title', main_file)
         self.assertIn('_audit_conform.dict_name', assoc_file)
         self.assertNotIn('_audit_conform.dict_name', main_file)
 
@@ -943,7 +951,7 @@ _ma_associated_archive_file_details.description
         f = modelcif.associated.CIFFile(
             path='/not/exist/foo.cif',
             local_path='test_write_associated_copy.cif',
-            categories=['exptl'], copy_categories=['entity', 'audit_conform'],
+            categories=['struct'], copy_categories=['entity', 'audit_conform'],
             entry_details='test details', entry_id='testcif')
         r = modelcif.associated.Repository(url_root='https://example.com',
                                            files=[f])
@@ -955,9 +963,9 @@ _ma_associated_archive_file_details.description
         with open('test_write_associated_copy.cif') as fh:
             assoc_file = fh.read()
         os.unlink('test_write_associated_copy.cif')
-        # exptl category should be in associated file, not the main file
-        self.assertIn('_exptl.entry_id', assoc_file)
-        self.assertNotIn('_exptl.entry_id', main_file)
+        # struct category should be in associated file, not the main file
+        self.assertIn('_struct.title', assoc_file)
+        self.assertNotIn('_struct.title', main_file)
         # entity and audit conform categories should be in *both* files
         self.assertIn('_entity.type', assoc_file)
         self.assertIn('_entity.type', main_file)
@@ -977,7 +985,7 @@ _ma_associated_archive_file_details.description
         fh = StringIO()
         modelcif.dumper.write(fh, [s])
         main_file = fh.getvalue()
-        self.assertIn('_exptl.entry_id', main_file)
+        self.assertIn('_struct.title', main_file)
         self.assertIn('_audit_conform.dict_name', main_file)
 
     @unittest.skipIf(msgpack is None, "needs Python 3 and msgpack")
@@ -987,7 +995,7 @@ _ma_associated_archive_file_details.description
 
         f = modelcif.associated.CIFFile(
             path='test_write_associated_binary.bcif',
-            categories=['exptl', '_AUDIT_CONFORM'],
+            categories=['struct', '_AUDIT_CONFORM'],
             entry_details='test details', entry_id='testcif', binary=True)
         r = modelcif.associated.Repository(url_root='https://example.com',
                                            files=[f])
@@ -1002,8 +1010,8 @@ _ma_associated_archive_file_details.description
         assoc_cats = frozenset(
             x['name'] for x in assoc_file['dataBlocks'][0]['categories'])
 
-        self.assertIn('_exptl', assoc_cats)
-        self.assertNotIn('_exptl.entry_id', main_file)
+        self.assertIn('_struct', assoc_cats)
+        self.assertNotIn('_struct.title', main_file)
         self.assertIn('_audit_conform', assoc_cats)
         self.assertNotIn('_audit_conform.dict_name', main_file)
 
@@ -1056,7 +1064,26 @@ _pdbx_entity_nonpoly.ma_model_mode
     def test_chem_comp_dumper(self):
         """Test ChemCompDumper"""
         system = modelcif.System()
-        e1 = modelcif.Entity('AC')
+
+        # Old-style ChemComp without ccd
+        c1 = ihm.NonPolymerChemComp('C1', name='C1')
+        if hasattr(c1, 'ccd'):
+            del c1.ccd
+
+        # ChemComp using core CCD
+        c2 = ihm.NonPolymerChemComp('C2', name='C2')
+        c2.ccd = 'core'
+
+        # ChemComp using MA CCD
+        c3 = ihm.NonPolymerChemComp('C3', name='C3')
+        c3.ccd = 'ma'
+
+        # ChemComp with descriptors (local)
+        c4 = ihm.NonPolymerChemComp('C4', name='C4')
+        c4.ccd = None
+        c4.descriptors = [modelcif.descriptor.IUPACName("foo")]
+
+        e1 = modelcif.Entity(['A', 'C', c1, c2, c3, c4])
         system.entities.append(e1)
 
         e2 = modelcif.Entity('GT')
@@ -1073,10 +1100,67 @@ _chem_comp.type
 _chem_comp.name
 _chem_comp.formula
 _chem_comp.formula_weight
-ALA 'L-peptide linking' ALANINE 'C3 H7 N O2' 89.094
-CYS 'L-peptide linking' CYSTEINE 'C3 H7 N O2 S' 121.154
-GLY 'peptide linking' GLYCINE 'C2 H5 N O2' 75.067
-THR 'L-peptide linking' THREONINE 'C4 H9 N O3' 119.120
+_chem_comp.ma_provenance
+ALA 'L-peptide linking' ALANINE 'C3 H7 N O2' 89.094 'CCD Core'
+C1 non-polymer C1 . . 'CCD Core'
+C2 non-polymer C2 . . 'CCD Core'
+C3 non-polymer C3 . . 'CCD MA'
+C4 non-polymer C4 . . 'CCD local'
+CYS 'L-peptide linking' CYSTEINE 'C3 H7 N O2 S' 121.154 'CCD Core'
+GLY 'peptide linking' GLYCINE 'C2 H5 N O2' 75.067 'CCD Core'
+THR 'L-peptide linking' THREONINE 'C4 H9 N O3' 119.120 'CCD Core'
+#
+""")
+
+    def test_chem_comp_dumper_bad_ccd(self):
+        """Test ChemCompDumper with invalid value for ccd"""
+        system = modelcif.System()
+
+        c1 = ihm.NonPolymerChemComp('C1', name='C1')
+        c1.ccd = 'garbage'
+
+        e1 = modelcif.Entity([c1])
+        system.entities.append(e1)
+
+        dumper = modelcif.dumper._ChemCompDumper()
+        self.assertRaises(KeyError, _get_dumper_output, dumper, system)
+
+    def test_chem_comp_descriptor_dumper(self):
+        """Test ChemCompDescriptorDumper"""
+        class MockObject(object):
+            pass
+
+        system = modelcif.System()
+
+        # Old-style ChemComp without descriptors
+        c1 = ihm.NonPolymerChemComp('C1', name='C1name')
+        if hasattr(c1, 'descriptor'):
+            del c1.descriptors
+
+        c2 = ihm.NonPolymerChemComp('C2', name='C2name')
+        c2.ccd = None
+        soft = MockObject()
+        soft._id = 42
+        c2.descriptors = [modelcif.descriptor.IUPACName("foo"),
+                          modelcif.descriptor.PubChemCID(
+                              "bar", details="test details", software=soft)]
+
+        e1 = modelcif.Entity(['A', 'C', c1, c2])
+        system.entities.append(e1)
+
+        dumper = modelcif.dumper._ChemCompDescriptorDumper()
+        out = _get_dumper_output(dumper, system)
+        self.assertEqual(out, """#
+loop_
+_ma_chem_comp_descriptor.ordinal_id
+_ma_chem_comp_descriptor.chem_comp_id
+_ma_chem_comp_descriptor.chem_comp_name
+_ma_chem_comp_descriptor.type
+_ma_chem_comp_descriptor.value
+_ma_chem_comp_descriptor.details
+_ma_chem_comp_descriptor.software_id
+1 C2 C2name 'IUPAC Name' foo . .
+2 C2 C2name 'PubChem CID' bar 'test details' 42
 #
 """)
 
