@@ -279,6 +279,55 @@ _ma_template_non_poly.details
         self.assertIs(a.template, t)
         self.assertTrue(a.explicit)
 
+    def test_entity_nonpoly_bad_model_mode(self):
+        """Test pdbx_entity_nonpoly with missing ma_model_mode"""
+        cif = """
+loop_
+_struct_asym.id
+_struct_asym.entity_id
+A 1
+B 2
+C 3
+#
+loop_
+_pdbx_entity_nonpoly.entity_id
+_pdbx_entity_nonpoly.name
+_pdbx_entity_nonpoly.comp_id
+_pdbx_entity_nonpoly.ma_model_mode
+1 test1 TE1 explicit
+2 test2 TE2 .
+3 test3 TE3 ?
+#
+loop_
+_ma_template_details.ordinal_id
+_ma_template_details.template_id
+_ma_template_details.template_origin
+_ma_template_details.template_entity_type
+_ma_template_details.template_trans_matrix_id
+_ma_template_details.template_data_id
+_ma_template_details.target_asym_id
+_ma_template_details.template_label_asym_id
+_ma_template_details.template_label_entity_id
+_ma_template_details.template_model_num
+_ma_template_details.template_auth_asym_id
+1 1 'reference database' non-polymer 1 2 A A 1 4 Z
+2 2 'reference database' non-polymer 1 3 B B 2 4 Z
+3 3 'reference database' non-polymer 1 4 C C 3 4 Z
+#
+loop_
+_ma_template_non_poly.template_id
+_ma_template_non_poly.comp_id
+_ma_template_non_poly.details
+1 TE1 test1
+2 TE2 test2
+3 TE3 test3
+"""
+        s, = modelcif.reader.read(StringIO(cif))
+        a1, a2, a3 = s.asym_units
+        self.assertTrue(a1.explicit)
+        self.assertIsNone(a2.explicit)
+        self.assertIs(a3.explicit, ihm.unknown)
+
     def test_template_ref_db_handler(self):
         """Test _TemplateRefDBHandler"""
         cif = """
@@ -287,19 +336,29 @@ _ma_template_ref_db_details.template_id
 _ma_template_ref_db_details.db_name
 _ma_template_ref_db_details.db_name_other_details
 _ma_template_ref_db_details.db_accession_code
-1 PDB . 3nc1
-1 MIS . testacc
-1 Other foo acc2
+_ma_template_ref_db_details.db_version_date
+1 PDB . 3nc1 2021-10-06
+1 MIS . testacc .
+1 Other foo acc2 .
+1 PubChem . 1046 .
+1 AlphaFoldDB . I6XD65 2022-06-01
 """
         s, = modelcif.reader.read(StringIO(cif))
         t, = s.templates
-        r1, r2, r3 = t.references
+        r1, r2, r3, r4, r5 = t.references
         self.assertIsInstance(r1, modelcif.reference.PDB)
         self.assertEqual(r1.accession, '3nc1')
+        self.assertEqual(r1.db_version_date, date(2021, 10, 6))
         self.assertEqual(r2.name, 'MIS')
         self.assertIsNone(r2.other_details)
+        self.assertIsNone(r2.db_version_date)
         self.assertEqual(r3.name, 'Other')
         self.assertEqual(r3.other_details, 'foo')
+        self.assertIsInstance(r4, modelcif.reference.PubChem)
+        self.assertEqual(r4.accession, '1046')
+        self.assertIsInstance(r5, modelcif.reference.AlphaFoldDB)
+        self.assertEqual(r5.accession, 'I6XD65')
+        self.assertEqual(r5.db_version_date, date(2022, 6, 1))
 
     def _get_models_cif(self):
         cif = """
@@ -420,6 +479,7 @@ _ma_struct_assembly.seq_id_begin
 _ma_struct_assembly.seq_id_end
 1 1 1 A 1 2
 2 1 1 A 1 1
+3 1 1 A . .
 #
 loop_
 _ma_struct_assembly_details.assembly_id
@@ -431,12 +491,14 @@ _ma_struct_assembly_details.assembly_description
         a, = s.assemblies
         self.assertEqual(a.name, 'foo')
         self.assertEqual(a.description, 'bar')
-        self.assertEqual(len(a), 2)
+        self.assertEqual(len(a), 3)
         # Complete asym
         self.assertIsInstance(a[0], modelcif.AsymUnit)
         # asym range
         self.assertIsInstance(a[1], modelcif.AsymUnitRange)
         self.assertEqual(a[1].seq_id_range, (1, 1))
+        # No specified range -> complete asym
+        self.assertIsInstance(a[2], modelcif.AsymUnit)
 
     def test_template_poly_segment_handler(self):
         """Test _TemplatePolySegmentHandler"""
@@ -806,6 +868,7 @@ _ma_alignment_info.alignment_type
 _ma_alignment_info.alignment_mode
 1 3 1 . 'target-template pairwise alignment' global
 2 4 1 . 'target-template pairwise alignment' global
+3 5 1 . 'target-template pairwise alignment' global
 #
 #
 loop_
@@ -820,6 +883,7 @@ _ma_alignment_details.percent_sequence_identity
 _ma_alignment_details.sequence_identity_denominator
 _ma_alignment_details.sequence_identity_denominator_other_details
 1 1 1 A 'BLAST e-value' . 1.0 45.000 'Length of the shorter sequence' .
+2 3 1 A 'HHblits e-value' . 2.0 45.000 'Length of the shorter sequence' .
 #
 loop_
 _ma_alignment.ordinal_id
@@ -828,6 +892,8 @@ _ma_alignment.target_template_flag
 _ma_alignment.sequence
 1 1 1 DSYV-ETLD
 2 1 2 DMACDTFIK
+3 1 1 DSYV-ETLD
+4 1 2 DMACDTFIK
 #
 loop_
 _ma_target_template_poly_mapping.id
@@ -836,9 +902,10 @@ _ma_target_template_poly_mapping.target_asym_id
 _ma_target_template_poly_mapping.target_seq_id_begin
 _ma_target_template_poly_mapping.target_seq_id_end
 1 1 A 1 8
+2 1 A 1 8
 """
         s, = modelcif.reader.read(StringIO(cif))
-        a1, a2, = s.alignments
+        a1, a2, a3, = s.alignments
         self.assertIs(a1.__class__, a2.__class__)
         self.assertIsInstance(a1, modelcif.alignment.Global)
         self.assertIsInstance(a1, modelcif.alignment.Pairwise)
@@ -856,6 +923,9 @@ _ma_target_template_poly_mapping.target_seq_id_end
         self.assertEqual(p.target.asym._id, 'A')
         self.assertEqual(p.target.gapped_sequence, 'DSYV-ETLD')
         self.assertEqual(p.target.seq_id_range, (1, 8))
+        p, = a3.pairs
+        self.assertIsInstance(p.score, modelcif.alignment.HHblitsEValue)
+        self.assertAlmostEqual(p.score.value, 2.0, delta=1e-6)
 
     def test_associated_files(self):
         """Test _AssociatedHandler and _AssociatedArchiveHandler"""
@@ -1008,6 +1078,60 @@ _ma_template_non_poly.details
         self.assertEqual(s1.type, 'non-polymer')
         self.assertIsInstance(s1, ihm.NonPolymerChemComp)
 
+    def test_chem_comp_handler(self):
+        """Test ChemCompHandler and ChemCompDescriptorHandler"""
+        cif = """
+loop_
+_chem_comp.id
+_chem_comp.type
+_chem_comp.name
+_chem_comp.formula
+_chem_comp.ma_provenance
+MET 'L-peptide linking' . . .
+CYS 'D-peptide linking' CYSTEINE . ?
+ALA 'L-peptide linking' ALANINE . 'CCD Core'
+MATYPE 'L-PEPTIDE LINKING' 'MODELARCHIVE COMPONENT' . 'CCD MA'
+MYTYPE 'L-PEPTIDE LINKING' 'MY CUSTOM COMPONENT' . 'CCD local'
+#
+loop_
+_ma_chem_comp_descriptor.ordinal_id
+_ma_chem_comp_descriptor.chem_comp_id
+_ma_chem_comp_descriptor.chem_comp_name
+_ma_chem_comp_descriptor.type
+_ma_chem_comp_descriptor.value
+_ma_chem_comp_descriptor.details
+_ma_chem_comp_descriptor.software_id
+1 MYTYPE 'ignored' 'InChI Key' XDAOLTSRNUSPPH-XMMPIXPASA-N foo 1
+2 MYTYPE ? 'IUPAC Name' foobar . .
+#
+loop_
+_entity_poly_seq.entity_id
+_entity_poly_seq.num
+_entity_poly_seq.mon_id
+_entity_poly_seq.hetero
+1 1 MET .
+1 2 CYS .
+1 3 ALA .
+1 4 MATYPE .
+1 5 MYTYPE .
+"""
+        s, = modelcif.reader.read(StringIO(cif))
+        e1, = s.entities
+        s = e1.sequence
+        self.assertEqual(len(s), 5)
+        self.assertEqual(s[2].ccd, 'core')
+        self.assertEqual(s[3].ccd, 'ma')
+        self.assertEqual(s[4].ccd, 'local')
+        d1, d2 = s[4].descriptors
+        self.assertIsInstance(d1, modelcif.descriptor.InChIKey)
+        self.assertEqual(d1.value, 'XDAOLTSRNUSPPH-XMMPIXPASA-N')
+        self.assertEqual(d1.details, 'foo')
+        self.assertEqual(d1.software._id, '1')
+        self.assertIsInstance(d2, modelcif.descriptor.IUPACName)
+        self.assertEqual(d2.value, 'foobar')
+        self.assertIsNone(d2.details)
+        self.assertIsNone(d2.software)
+
 
 if __name__ == '__main__':
     unittest.main()
